from django.urls import path
from . import views



urlpatterns = [
    path('', views.calendar),
    path('<user>', views.view_page, name='view_page'),
    path('get/<id>', views.get_data, name='get_data'),
    path('delete/<id>', views.delete_data, name='delete_data'),
    path('save/<id>', views.save_data, name='save_data'),
]


from django.shortcuts import render
from datetime import date, timedelta, datetime
from .models import Activity
import json
from django.http import HttpResponse, HttpResponseNotFound
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User


def convert_data(activities_object):
    """
    get result of query and translate it serializable format
    """
    data = []
    for i in activities_object:
        data.append({'id':i.id, 'title':i.__str__(), 'color':i.color, 'date':str(i.date.day),
            'start_time':str(int(str(i.start_time)[0:2])), 'finish_time':str(int(str(i.finish_time)[0:2]))})
    return data


@login_required()
def calendar(request):
    first_day = date.today().weekday()
    first_day = timedelta(days=first_day)
    first_day = date.today() - first_day
    way = [-1,1]
    try:
        way[0] = int(request.GET['n']) - 1
        way[1] = int(request.GET['n']) + 1
        delta = timedelta(days=int(request.GET['n']) * 7)
        first_day = first_day + delta
    except:
        pass
    # get activities between monday and sunday
    activities = Activity.objects.filter(user=request.user, date__range=(first_day, first_day+timedelta(days=6)))
    activities = convert_data(activities)   # get this data with js for draw activies on table
    week = []
    year = first_day.year
    month = first_day.month
    for i in range(7):
        delta = timedelta(days=i)
        week.append(datetime.strftime((first_day+delta), "%d-%B"))
    return render(request, "Calendar/WeaklyCalendar.html", {'activities' : activities,'week' : week,
                            'year' : year, 'range7' : range(7), 'range24' : range(24), 'month' : month, 'way' : way})


def view_page(request, user):
    first_day = date.today().weekday()
    first_day = timedelta(days=first_day)
    first_day = date.today() - first_day
    way = [-1,1]
    try:
        way[0] = int(request.GET['n']) - 1
        way[1] = int(request.GET['n']) + 1
        delta = timedelta(days=int(request.GET['n']) * 7)
        first_day = first_day + delta
    except:
        pass

    try:
        user = User.objects.get(username=user)
        activities = Activity.objects.filter(user=user, private=False, date__range=(first_day, first_day + timedelta(days=6)))
    except:
        return HttpResponseNotFound()

    activities = convert_data(activities)   # get this data with js for draw activies on table
    week = []
    year = first_day.year
    month = first_day.month
    for i in range(7):
        delta = timedelta(days=i)
        week.append(datetime.strftime((first_day+delta), "%d-%B"))
    return render(request, "Calendar/ReadCalendar.html", {'activities' : activities,'week' : week,
                            'year' : year, 'range7' : range(7), 'range24' : range(24), 'month' : month, 'way' : way})




@login_required()
def get_data(request, id):
    if(request.method != 'GET'):
        return HttpResponseNotFound()
    try:
        data = Activity.objects.get(id=id)
        data = Activity.objects.filter(title=data.title, start_time=data.start_time,
                                      finish_time=data.finish_time).first()
    except:
        return HttpResponseNotFound()
    data = {'id':data.id, 'title':data.title, 'date':data.date.__str__(), 'start_time':data.start_time.__str__(),
            'finish_time':data.finish_time.__str__(), 'repeat_fre':data.repeat_fre, 'repeat_time':data.repeat_time,
            'location':data.location, 'color':data.color, 'private':data.private, 'comment':data.comment}
    return HttpResponse(json.dumps(data), content_type="application/json")


@login_required()
def delete_data(request, id):
    if(request.method != 'GET'):
        return HttpResponseNotFound()
    try:
        Activity.objects.get(id=id).delete()
        return HttpResponse()
    except:
        return HttpResponseNotFound()


@login_required()
def save_data(request, id):
    # id = 0 means new activity
    if(id != "0"):
        try:
            act = Activity.objects.get(id=id, user_id=request.user.id)
        except:
            return HttpResponseNotFound()
    else:
        act = Activity(user_id=request.user.id)

    act.title = request.POST['title']
    act.date = datetime.strptime(request.POST.get('date'), '%Y-%m-%d')
    act.start_time = datetime.strptime(request.POST.get('start_time'), '%H:%M:%S')
    act.finish_time = datetime.strptime(request.POST.get('finish_time'), '%H:%M:%S')
    act.location = request.POST['location']
    act.color = request.POST['color']
    act.private = request.POST['private'] == 'true'
    act.comment = request.POST['comment']
    act.repeat_fre = request.POST['repeat_fre']
    act.repeat_time = int(request.POST['repeat_time'])
    act.save()

    return HttpResponse(json.dumps({'success': True}),content_type="application/json")



from django.db import models
from django.core.validators import MaxValueValidator
from django.contrib.auth.models import User
from datetime import date, timedelta, datetime
from dateutil.relativedelta import relativedelta

REPEAT_FREQ = (
    ('1', "Day"),
    ('2', "Week"),
    ('3', "Month"),
    ('4', "Year")
)

COLORS = [
    ('#eeeeee', 'gray'),
    ('#ff0000', 'red'),
    ('#0000ff', 'blue'),
    ('#00ff00', 'green'),
    ('#000000', 'black'),
    ('#ffffff', 'white'),
    ('#7E8F7C', 'khaki'),
]

class Activity(models.Model):
    title = models.CharField(verbose_name="Title", max_length=50)
    date = models.DateField(verbose_name="Date")
    start_time = models.TimeField(verbose_name="Start Time")
    finish_time = models.TimeField(verbose_name="Finish Time")
    repeat_fre = models.CharField(verbose_name="Repeat Frequency", max_length=50, choices=REPEAT_FREQ, blank=True)
    repeat_time = models.PositiveIntegerField(verbose_name="Repeat Time", validators=[MaxValueValidator(20)], blank=True, null=True)
    location = models.CharField(verbose_name="Location", max_length=50, blank=True, null=True)
    color = models.CharField(verbose_name="Color", max_length=7, choices=COLORS, default="gray")
    private = models.BooleanField(verbose_name="Private", default=False)
    comment = models.TextField(verbose_name="Comment", blank=True, null=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    related_activity = models.ManyToManyField('Activity', verbose_name="Related Activities", blank=True)

    def __str__(self):
        return self.title + " " + str(self.start_time) + "-" + str(self.finish_time)

    def save(self, *args, **kwargs):
        if(self.id != None):
            for i in self.related_activity.all():
                i.delete()

        if(self.repeat_fre == ''):
            return super().save(*args, **kwargs)
        super().save(*args, **kwargs)
        obj = Activity.objects.get(id=self.id)
        obj.repeat_time = 0
        obj.repeat_fre = 0
        for i in range(1,int(self.repeat_time)):
            obj.id = None;
            if (self.repeat_fre == "1"):    #day
                date = self.date + relativedelta(days=i)
            elif (self.repeat_fre == "2"):  #week
                date = self.date + relativedelta(days=7*i)
            elif (self.repeat_fre == "3"):  #month
                date = self.date + relativedelta(months=i)
            else:                           #year
                date = self.date + relativedelta(years=i)
            obj.date = date
            obj.save()
            # Following line does not work on admin panel bacause of
            # https://stackoverflow.com/questions/1925383/issue-with-manytomany-relationships-not-updating-immediately-after-save/1925784#1925784
            self.related_activity.add(obj)

    def delete(self, using=None, keep_parents=False):
        # if there are related activities
        if(self.related_activity.count()):
            for i in self.related_activity.all():
                i.delete()
            return super().delete()
        return  super().delete()


    class Meta:
        verbose_name = "Activity"
        verbose_name_plural = "Activities"



from  setuptools import setup, find_packages
import os

with open("README.md", "r") as fh:
    README = fh.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-weeklycalendar',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    license='GPL License',  
    description='Weekly Calendar for Django',
    long_description=README,
    url='https://github.com/yvznvr/WeeklyCalendar',
    author='Yavuz Unver',
    author_email='yvzunver@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
